import { App, Modal } from "obsidian";
import { mount, unmount, type Component, type MountOptions } from "svelte";

export class SvelteModal<
	TComponent extends Component<TProps, TExports, TBindings>,
	TProps extends Record<string, any> = {},
	TExports extends Record<string, any> = {},
	TBindings extends keyof TProps | "" = string
> extends Modal {
	protected component: TExports | undefined;

	constructor(
		app: App,
		private readonly componentCtor: TComponent,
		private readonly mountOpts: Omit<MountOptions<TProps>, "target">
	) {
		super(app);
	}

	onOpen(): void {
		this.component = mount(this.componentCtor, {
			...this.mountOpts,
			target: this.contentEl,
		});
	}

	onClose(): void {
		if (this.component) {
			unmount(this.component);
			this.component = undefined;
		}
	}
}
