import type BookTrackerPlugin from "@src/main";
import { writable, type Writable } from "svelte/store";
import { type BookTrackerSettings, DEFAULT_SETTINGS } from "./types";

type SettingsStore = Writable<BookTrackerSettings> & {
	load: () => Promise<void>;
};

export function createSettingsStore(plugin: BookTrackerPlugin): SettingsStore {
	const { subscribe, set, update } =
		writable<BookTrackerSettings>(DEFAULT_SETTINGS);

	async function load() {
		const settings = await plugin.loadData();

		update((currentSettings) => {
			return {
				...currentSettings,
				...settings,
			};
		});
	}

	subscribe((settings) => {
		if (settings === DEFAULT_SETTINGS) {
			return;
		}

		plugin.settings = settings;
		plugin.saveSettings();
	});

	return {
		subscribe,
		set,
		update,
		load,
	};
}
